import javax.swing.*; 
import javax.swing.text.*; 
import java.awt.*;
import java.text.*;
import java.util.*;


/**
 * A classe <code>CampoNumeroInteiro</code> representa um campo de texto com
 * entrada restrita a nmeros inteiros.
 * @author Grupo 4 - DOS
 * @version 1.0
 */
public class CampoNumeroInteiro extends JTextField {

    /** Fornece as configuraes locais. */
    private Toolkit tk;

    /** Formatador de nmeros. */
    private NumberFormat formatador;

    /**
     * Cria um novo objeto <code>CampoNumeroInteiro</code>.
     */
    public CampoNumeroInteiro() {
    
        this( 5 );
    }

    /**
     * Cria um novo objeto <code>CampoNumeroInteiro</code> com o nmero de 
     * colunas especificado.
     * @param colunas nmero de colunas do campo de texto
     */
    public CampoNumeroInteiro( int colunas ) {
        super( colunas );
        tk = Toolkit.getDefaultToolkit();
        formatador = NumberFormat.getNumberInstance( Locale.US );
        formatador.setParseIntegerOnly( true );
    }

    /**
     * Cria um novo objeto <code>CampoNumeroInteiro</code> com o valor e o 
     * nmero de colunas especificados.
     * @param valor valor inicial do campo
     * @param colunas nmero de colunas do campo de texto
     */
    public CampoNumeroInteiro( int valor, int colunas ) {
        super( colunas );
        tk = Toolkit.getDefaultToolkit();
        formatador = NumberFormat.getNumberInstance( Locale.US );
        formatador.setParseIntegerOnly( true );
        setValue( valor );
    }

    /**
     * Retorna o valor do campo de nmero inteiro.
     * @return o valor do campo de nmero inteiro.
     */
    public int getValue() {

        int valorDeRetorno = 0;
        try {
            valorDeRetorno = formatador.parse( getText() ).intValue();
        } catch ( ParseException e ) { }

        return valorDeRetorno;
    }

    /**
     * Altera o valor do campo de nmero inteiro para o novo valor especificado.
     * @param valor novo valor do campo
     */ 
    public void setValue( int valor ) {

        setText( formatador.format( valor ) );
    }

    /**
     * Apaga o valor do campo de nmero inteiro.
     */
    public void setValue() {

        setText("");
    }

    /**
     * Devolve um modelo padro para o campo de texto.
     * @return um modelo padro para o campo de texto.
     */
    protected Document createDefaultModel() {
        
	return new WholeNumberDocument();
    }


    /**
     * A classe <code>WholeNumberDocument</code> cuida da formatao do campo de
     * texto para aceitar apenas nmeros.
     */
    protected class WholeNumberDocument extends PlainDocument {
        
	/**
	 * Trata a entrada recebida no campo de texto.
	 * @param offs deslocamento inicial >= 0 
	 * @param str "string" a ser inserida; no faz nada com "strings" vazios
	 * @param a atributos para o contedo inserido
	 */
	public void insertString( int offs, String str, AttributeSet a ) 
	    throws BadLocationException {
            
	    char[] fonte = str.toCharArray();
            char[] resultado = new char[ fonte.length ];
            int j = 0;

            for ( int i = 0; i < resultado.length; i++ ) {
                if ( Character.isDigit( fonte[ i ] ) )
                    resultado[ j++ ] = fonte[ i ];
                else {
		    tk.beep();
                }
            }
            super.insertString( offs, new String( resultado, 0, j ), a );
        }
    }
}







