import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;
import java.util.Collection;
import java.util.Collections;

/**
 * 
 * @version 1.0
 * @author Alex Minoru Kusano & Marcos Roberto Yukio Koga
 */

public class Gerenciador implements Observador, Observado {

    private ArrayList jogadores;
    private FabricaDeUnidades fabrica;
    private Setor[][] mapa;

    private HashMap acoes;

    private ArrayList ordem;

    private int jogadorAtual;

    /** colecao de observadores */
    private ArrayList observadores;

    private static Gerenciador instancia;


    private Gerenciador() {
	
	jogadores = new ArrayList();
        fabrica = FabricaDeUnidades.recuperaInstancia();
        acoes = new HashMap();
	observadores = new ArrayList();
        incluiAcoes();        
    }

    private void incluiAcoes() {
        
        AcaoAbstrata acao;
        acao =  new Atacar();
        acoes.put( acao.recuperaNome(), acao );
        acao =  new Cegar();
        acoes.put( acao.recuperaNome(), acao );
        acao = new Bomba();
        acoes.put( acao.recuperaNome(), acao );
        acao = new Paulada();
        acoes.put( acao.recuperaNome(), acao );
	acao = new Bola();
        acoes.put( acao.recuperaNome(), acao );
	acao = new Dardo();
        acoes.put( acao.recuperaNome(), acao );
	acao = new BolaDeFogo();
        acoes.put( acao.recuperaNome(), acao );
	acao = new TempestadeDeGelo();
        acoes.put( acao.recuperaNome(), acao );
	acao = new Drenar();
        acoes.put( acao.recuperaNome(), acao );
	acao = new Socar();
        acoes.put( acao.recuperaNome(), acao );
	acao = new AtaquePoderoso();
        acoes.put( acao.recuperaNome(), acao );
    }

    public static Gerenciador recuperaInstancia() {
        
        if( instancia == null ){
            instancia = new Gerenciador();
        }
        return instancia;
    }

    public Setor[][] geraMapa() {
        
        return geraMapa( 20, 20 );
    }
    
    public Setor[][] geraMapa( int n, int m ) {
        
        mapa = new Setor[n][m];
        for( int i = 0; i < n; i++ ) {
            for( int j = 0; j < m; j++ ) {
                mapa[i][j] = new Setor( j, i );
            }
        }
        return mapa;
    }

    public Setor[][] geraMapa( int[][] custos, String[][] figuras ) {
        
        mapa = new Setor[custos.length][custos[0].length];
        for( int i = 0; i < custos.length; i++ ) {
            for( int j = 0; j < custos[i].length; j++ ) {
                mapa[i][j] = new Setor( figuras[i][j], custos[i][j], j, i );
            }
        }
        return mapa;
    }

    public Setor[][] recuperaMapa() {
        
        return mapa;
    }

    public void alteraMapa( int x, int y, int novoCusto, String novoFundo ) {
        
        if( mapa != null ) {
            mapa[y][x].alteraCustoDeMovimento( novoCusto );
            mapa[y][x].alteraFundo( novoFundo );
        }
    }

    public void novoJogador() {
	
	String nome = "jogador" + Integer.toString( jogadores.size() + 1 );
	Jogador jogador = new Jogador( nome );
	jogadores.add( jogador );
	jogador.adicionaObservador( this );
	
    }

    public void criaUnidade( String nomeUnidade, int numeroJogador ) {
    
        Jogador jogador = ( Jogador ) jogadores.get( numeroJogador );
        if( jogador != null ) {
            Unidade unidade = fabrica.criaUnidade( nomeUnidade );
	    unidade.alteraJogador( jogador );
            jogador.adicionaUnidade( unidade );
	    unidade.adicionaObservador( this );
        }
    }

    public Unidade[] recuperaPrototipos() {
        
        return fabrica.recuperaPrototipos();
    }

    public Unidade[] recuperaUnidades( int numeroJogador ) {
        
        Jogador jogador = ( Jogador ) jogadores.get( numeroJogador );
        if( jogador != null ){
            return jogador.recuperaUnidades();
        }
        return null;
    }

    public AcaoAbstrata recuperaAcao( String nomeDaAcao ) {
        
        return ( AcaoAbstrata )acoes.get( nomeDaAcao );
    }
    
    public AcaoAbstrata[] recuperaAcoes() {
        
        AcaoAbstrata[] retorno = new AcaoAbstrata[ acoes.size() ];
	acoes.values().toArray( retorno );
	return retorno;
    }


    public void moveUnidade( Unidade unidade, int x, int y ) {
        
        Setor antigo = unidade.recuperaPosicao();
        
        antigo.removeUnidade();
        unidade.move( mapa[y][x] );
        mapa[y][x].alteraUnidade( unidade );
    }

    public void executaAcao( Unidade unidade, 
                             AcaoAbstrata acao, 
                             int x, int y ) {
        
        unidade.executaAcao( acao );
        Iterator iterador = acao.calculaArea().iterator();

        while( iterador.hasNext() ) {
            Coordenada coordenada = ( Coordenada ) iterador.next();

            int ajusteX = coordenada.recuperaX();
            int ajusteY = coordenada.recuperaY();
            
            if( y+ajusteY >= 0 && y+ajusteY < mapa.length &&
                x+ajusteX >= 0 && x+ajusteX < mapa[0].length ) {
                Unidade unidadeAfetada = 
                    mapa[y+ajusteY][x+ajusteX].recuperaUnidade();
                if( unidadeAfetada != null ) {
                    unidadeAfetada.sofreAcao( acao );
                }
            }
        }
    }

    public void atualiza( Object objeto ) {

	Unidade unidade;

	try {
	    unidade = (Unidade) objeto;

	} catch( ClassCastException cce ) {

	    try {
		Jogador jogador = (Jogador) objeto;
		
		System.out.println( "O " + jogador.recuperaNome() + 
				    " est fora do jogo." );
		Integer numero = new Integer( jogadores.indexOf( jogador ) );
		int indiceSaiu = ordem.indexOf( numero );
		if( indiceSaiu <= jogadorAtual && jogadorAtual > 0)
		    jogadorAtual--;
		ordem.remove( numero );

	    } catch( ClassCastException e ) { }

	    return;
	}

	notificaObservadores( unidade );
    }

    public void adicionaObservador( Observador observador ) {
	
	observadores.add( observador );
    }

    public void removeObservador( Observador observador ) {
    
	int i = observadores.indexOf( observador );
	if( i != -1 ) {
	    observadores.remove( i );
	}
    }

    protected void notificaObservadores( Object estado ) {
        
	Observador observador;
	for( int i = 0; i < observadores.size(); i++ ) {
	    observador = ( Observador ) observadores.get( i );
	    observador.atualiza( estado );
	}
    }

    public void criaPrototipo( String nome, 
                               int pontosDeVida,
                               int precisao,
                               int pontosDeMovimento,
			       String figura,
                               AcaoAbstrata[] acoes ) {
        Unidade novaUnidade = new Unidade( nome,
                                           pontosDeVida, 
                                           precisao,
                                           pontosDeMovimento,
					   figura );
        for( int i = 0; i < acoes.length; i++ ) {
            novaUnidade.adicionaAcao( acoes[i] );
        }
        fabrica.adicionaPrototipo( nome, novaUnidade );
    }

    public void criaPrototipo( Unidade novaUnidade ) {
        
        fabrica.adicionaPrototipo( novaUnidade.recuperaNome(),
                                   novaUnidade );
    }

    /**
     * Cria um novo jogo para o nmero de jogadores especificado.
     */
    public void novoJogo( int numeroDeJogadores ) {

	for( int i = 0; i < numeroDeJogadores; i++ ) {
	    this.novoJogador();
	}

	System.out.println("Criando novo jogo para " + numeroDeJogadores +
			   " jogadores...");

	// Sorteia a ordem dos jogadores
	Random dado = new Random();
        ArrayList aux = new ArrayList();

	for( int i = 0; i < numeroDeJogadores; i++ ) {
	    aux.add( new Integer( i ) );
        }

	ordem = new ArrayList( numeroDeJogadores );
        for( int i = 0; i < numeroDeJogadores; i++ ) {
	    int prox = dado.nextInt( aux.size() );
	    ordem.add( (Integer) aux.remove( prox ) );
        }

	jogadorAtual = 0; 
    }

    /**
     * Devolve o nmero do prximo jogador a executar movimentos.
     */
    public int proximoJogador() {

	if( ordem.size() == 1 ) {
	    Jogador vencedor = (Jogador)
		jogadores.get( ((Integer) ordem.get(0)).intValue() );
	    System.out.println( "Parabns, " + vencedor.recuperaNome() +
				". Voc ganhou o jogo!!!" );
	    System.exit(0);
	}

	if( jogadorAtual == ordem.size() ) {
	    jogadorAtual = 0;
	}

	return ((Integer) ordem.get(jogadorAtual++)).intValue();
    }


    /**
     * Define o nmero de unidades para um jogador.
     */
    public void alteraTotalDeUnidades( int jogador, int numeroUnidades ) {

	( ( Jogador )jogadores.get( jogador )).
	    alteraTotalDeUnidades( numeroUnidades );
    }

    /**
     * Define o nmero de unidades para todos os jogadores.
     */
    public void alteraTotalDeUnidades( int numeroUnidades ) {

	for( int i = 0; i < jogadores.size(); i++ ) {
	    (( Jogador )jogadores.get( i ) ).
		alteraTotalDeUnidades( numeroUnidades );
	}
    }

    public Jogador[] recuperaJogadores() {

        Jogador[] retorno = new Jogador[ jogadores.size() ];
        
        jogadores.toArray( retorno );
        return retorno;
    }

    public int recuperaTotalJogadores() {

	return jogadores.size();
    }

    public Collection recuperaPosicoes( Unidade unidade, AcaoAbstrata acao ) {
        
        Setor setor = unidade.recuperaPosicao();
        Coordenada coord = new Coordenada( setor.recuperaX(), 
					   setor.recuperaY() );
	int alcance = acao.recuperaAlcanceDaAcao();

        ArrayList listaSetores = new ArrayList();
	ArrayList aux = new ArrayList();

	listaSetores.add( new ItemDaLista( coord, 0 ) );
	while( listaSetores.size() > 0 ) {
            
            ItemDaLista item = (ItemDaLista) listaSetores.remove(0);

	    aux.add( item );
	    
	    int valX = item.recuperaCoordenada().recuperaX();
	    int valY = item.recuperaCoordenada().recuperaY();
	    int custo = item.recuperaCusto();

            ItemDaLista novoItem;
	    if( valX - 1 >= 0 ) {
		novoItem = 
		    new ItemDaLista( 
			   new Coordenada( valX-1, valY ), 
			   custo + 1 );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= alcance ) {
		    listaSetores.add( novoItem );
		}

		if( valY - 1 >= 0 ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX-1, valY-1), 
			   custo + 1 );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= alcance ) {
			listaSetores.add( novoItem );
		    }
		}

		if( valY + 1 < mapa.length ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX-1, valY+1 ), 
			   custo + 1 );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= alcance ) {
			listaSetores.add( novoItem );
		    }
		}
	    }

	    if( valX + 1 < mapa[0].length ) {
		novoItem = 
		    new ItemDaLista( 
			   new Coordenada( valX+1, valY ), 
			   custo + 1 );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= alcance ) {
		    listaSetores.add( novoItem );
		}

		if( valY - 1 >= 0 ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX+1, valY-1 ), 
			   custo + 1 );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= alcance ) {
			listaSetores.add( novoItem );
		    }
		}

		if( valY + 1 < mapa.length ) {
		    novoItem = 
			new ItemDaLista( new Coordenada( valX+1, valY+1 ), 
                                         custo + 1 );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= alcance ) {
			listaSetores.add( novoItem );
		    }
		}
	    }

	    if( valY - 1 >= 0 ) {
		novoItem = new ItemDaLista( new Coordenada( valX, valY-1 ), 
		    custo + 1 );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= alcance ) {
		    listaSetores.add( novoItem );
		}
	    }

	    if( valY + 1 < mapa.length ) {
		novoItem = new ItemDaLista( new Coordenada( valX, valY+1 ), 
                                            custo + 1 );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= alcance ) {
		    listaSetores.add( novoItem );
		}
	    }
        }
        
        ArrayList retorno = new ArrayList();
	for( int i = 0; i < aux.size(); i++ ) {
	    retorno.add( ((ItemDaLista) aux.get(i)).recuperaCoordenada() );
	}
        return retorno;
    }

    public Collection recuperaPosicoes( Unidade unidade ) {

	Setor setor = unidade.recuperaPosicao();
	Coordenada coord = new Coordenada( setor.recuperaX(), 
					   setor.recuperaY() );
	int pontosMovimento = unidade.recuperaPontosDeMovimento();

	ArrayList listaSetores = new ArrayList();
	ArrayList aux = new ArrayList();

	listaSetores.add( new ItemDaLista( coord, 0 ) );
	while( listaSetores.size() > 0 ) {

	    ItemDaLista item = (ItemDaLista) listaSetores.remove(0);

	    aux.add( item );
	    
	    int valX = item.recuperaCoordenada().recuperaX();
	    int valY = item.recuperaCoordenada().recuperaY();
	    int custo = item.recuperaCusto();
	    ItemDaLista novoItem;
	    if( valX - 1 >= 0 ) {
		novoItem = 
		    new ItemDaLista( 
			   new Coordenada( valX-1, valY ), 
			   custo + 
			   mapa[valY][valX-1].recuperaCustoDeMovimento() );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= pontosMovimento &&
		    mapa[valY][valX-1].estaVazio() ) {
		    listaSetores.add( novoItem );
		}

		if( valY - 1 >= 0 ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX-1, valY-1), 
			   custo + 
			   mapa[valY-1][valX-1].recuperaCustoDeMovimento() );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= pontosMovimento &&
			mapa[valY-1][valX-1].estaVazio() ) {
			listaSetores.add( novoItem );
		    }
		}

		if( valY + 1 < mapa.length ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX-1, valY+1), 
			   custo + 
			   mapa[valY+1][valX-1].recuperaCustoDeMovimento() );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= pontosMovimento &&
			mapa[valY+1][valX-1].estaVazio() ) {
			listaSetores.add( novoItem );
		    }
		}
	    }

	    if( valX + 1 < mapa[0].length ) {
		novoItem = 
		    new ItemDaLista( 
			   new Coordenada( valX+1, valY ), 
			   custo + 
			   mapa[valY][valX+1].recuperaCustoDeMovimento() );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= pontosMovimento &&
		    mapa[valY][valX+1].estaVazio() ) {
		    listaSetores.add( novoItem );
		}

		if( valY - 1 >= 0 ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX+1, valY-1), 
			   custo + 
			   mapa[valY-1][valX+1].recuperaCustoDeMovimento() );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= pontosMovimento &&
		    mapa[valY-1][valX+1].estaVazio() ) {
			listaSetores.add( novoItem );
		    }
		}

		if( valY + 1 < mapa.length ) {
		    novoItem = 
			new ItemDaLista( 
			   new Coordenada( valX+1, valY+1), 
			   custo + 
			   mapa[valY+1][valX+1].recuperaCustoDeMovimento() );

		    if( !aux.contains(novoItem) && 
			!listaSetores.contains(novoItem) &&
			novoItem.recuperaCusto() <= pontosMovimento &&
		    mapa[valY+1][valX+1].estaVazio() ) {
			listaSetores.add( novoItem );
		    }
		}
	    }

	    if( valY - 1 >= 0 ) {
		novoItem = new ItemDaLista( new Coordenada( valX, valY-1), 
		    custo + mapa[valY-1][valX].recuperaCustoDeMovimento() );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= pontosMovimento  &&
		    mapa[valY-1][valX].estaVazio() ) {
		    listaSetores.add( novoItem );
		}
	    }

	    if( valY + 1 < mapa.length ) {
		novoItem = new ItemDaLista( new Coordenada( valX, valY+1), 
		   custo + mapa[valY+1][valX].recuperaCustoDeMovimento() );

		if( !aux.contains(novoItem) && 
		    !listaSetores.contains(novoItem) &&
		    novoItem.recuperaCusto() <= pontosMovimento &&
		    mapa[valY+1][valX].estaVazio() ) {
		    listaSetores.add( novoItem );
		}
	    }
	
	    Collections.sort( listaSetores );
	}

	ArrayList retorno = new ArrayList();
	for( int i = 0; i < aux.size(); i++ ) {
	    retorno.add( ((ItemDaLista) aux.get(i)).recuperaCoordenada() );
	}

	return retorno;
    }


    private class ItemDaLista implements Comparable {

	private Coordenada coordenada;
	private int custo;

	public ItemDaLista( Coordenada coordenada, int custo ) {

	    this.coordenada = coordenada;
	    this.custo = custo;
	}

	public boolean equals( Object obj ) {

	    if( getClass().isInstance(obj) ) {
		return( coordenada.equals( 
			  ((ItemDaLista) obj).recuperaCoordenada() ) );
	    }

	    return false;
	}

	public int compareTo( Object obj ) {

	    if( getClass().isInstance(obj) ) {
		return( custo - ((ItemDaLista) obj).recuperaCusto() );
	    }

	    return 1;
	}

	public int recuperaCusto() {
	    
	    return custo;
	}

	public Coordenada recuperaCoordenada() {

	    return coordenada;
	}
    }

    public static void main( String[] args ){

        JanelaNumeroJogadores jt = new JanelaNumeroJogadores();
        jt.pack();
        jt.show();
        
        /*
	Unidade[] unidades;
        java.util.Random dado = new java.util.Random();
	String[] nomes = { "unidade 1", "unidade 2" };

        AcaoAbstrata[] acoes = { new Atacar(), 
                                 new Cegar() };
        
        Gerenciador gerenciador = Gerenciador.recuperaInstancia();
        
        gerenciador.criaPrototipo( "teste",
                                   250, 20, 5,
				   null,
                                   acoes );

        gerenciador.novoJogador();

        gerenciador.alteraTotalDeUnidades( 0, 2 );

        gerenciador.criaUnidade( "teste", 0 );
        gerenciador.criaUnidade( "teste", 0 );
        
	unidades = gerenciador.recuperaUnidades( 0 );      

        acoes = unidades[0].recuperaAcoes();

	int primeiro, segundo, k;
	while( unidades[0].recuperaPontosDeVida() > 0 &&
	       unidades[1].recuperaPontosDeVida() > 0 ) {
	    
	    primeiro = dado.nextInt( 2 );
	    if( primeiro == 0 ) segundo = 1;
	    else segundo = 0;

	    k = dado.nextInt( 100 );
	    if( k < 15 ) {
		System.out.println( nomes[primeiro] + " - cegar " );
		acoes[1].associaAgente( unidades[primeiro] );
		unidades[segundo].sofreAcao( acoes[1] );
	    }
	    else {
		System.out.println( nomes[primeiro] + " - atacar " );
		acoes[0].associaAgente( unidades[primeiro] );
		unidades[segundo].sofreAcao( acoes[0] );
	    }
	    k = dado.nextInt( 100 );
	    if( k < 15 ) {
		System.out.println( nomes[segundo] + " - cegar " );
		acoes[1].associaAgente( unidades[segundo] );
		unidades[primeiro].sofreAcao( acoes[1] );
	    }
	    else {
		System.out.println( nomes[segundo] + " - atacar " );
		acoes[0].associaAgente( unidades[segundo] );
		unidades[primeiro].sofreAcao( acoes[0] );
	    }
	    
	    System.out.println( "\nresultado do round: \nunidade 1 " + 
	      Integer.toString( unidades[0].recuperaPontosDeVida() ) +
                                                               " - " + 
                  Integer.toString( unidades[0].recuperaPrecisao() ) +
                                                      "\nunidade 2 " +
	      Integer.toString( unidades[1].recuperaPontosDeVida() ) +
                                                               " - " + 
                  Integer.toString( unidades[1].recuperaPrecisao() ) +
                                                                "\n"   );
	}
        */
    }
}








