import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class JanelaCriacaoMapa extends JFrame implements Observador {
    
    private CampoNumeroInteiro x;

    private CampoNumeroInteiro y;

    private CampoNumeroInteiro custo;
    
    private JComboBox fundo;

    private Setor[][] mapa;

    private PainelControleMapa pcm;

    private JFileChooser jfc;

    public JanelaCriacaoMapa( int linhas, int colunas ) {
        
        super( "Criao de Mapa" );
        inicializaComponentes( linhas, colunas );
        
        inicializaMenu();
        
        addWindowListener( new WindowAdapter() {	
		public void windowClosing( WindowEvent we ) {                 
                    System.exit(0);
                }
	    } );
    }

    private void inicializaComponentes( int linhas, int colunas ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( linhas, colunas );
        
        pcm = new PainelControleMapa( mapa );
        pcm.setBorder( BorderFactory.createEtchedBorder() );
        pcm.adicionaObservador( this );

        ArrayList permitidos = new ArrayList();
        for( int i = 0; i < linhas; i++ ) {
            for( int j = 0; j < colunas; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }
        pcm.alteraPermitidos( permitidos );

        JPanel painelControle = new JPanel( new GridBagLayout() );
        painelControle.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        GridBagConstraints gbc = new GridBagConstraints();

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridx = 0;
        gbc.weightx = 1;
        gbc.gridy = 0;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.NORTH;

        JPanel aux = new JPanel( new GridLayout( 2, 1 ) );

        aux.add( new JLabel( "x:" ) );
        x = new CampoNumeroInteiro( 3 );
        aux.add( x );
        painelControle.add( aux, gbc );

        gbc.gridy = 1;
        
        aux = new JPanel( new GridLayout( 2, 1 ) );
        
        aux.add( new JLabel( "y:" ) );
        y = new CampoNumeroInteiro( 3 );
        aux.add( y );
        painelControle.add( aux, gbc );

        gbc.gridy = 2;

        aux = new JPanel( new GridLayout( 2, 1 ) );
        
        aux.add( new JLabel( "custo:" ) );
        custo = new CampoNumeroInteiro( 3 );
        aux.add( custo );
        painelControle.add( aux, gbc );
        
        gbc.gridy = 3;

        aux = new JPanel( new GridLayout( 2, 1 ) );

        File imagens = new File( "imagens" + File.separator + "fundos" );
        if( imagens.isDirectory() ) {
            fundo = new JComboBox( imagens.list() );
        }
        else {
            fundo = new JComboBox();
        }
        fundo.addItem( "" );
        fundo.setSelectedItem( "" );
        
        aux.add( new JLabel( "fundo:" ) );
        
        aux.add( fundo );
        painelControle.add( aux, gbc );
        
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridy = 4;
        gbc.weighty = 4;
        painelControle.add( new JPanel(), gbc );

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridx = 0;
        gbc.weightx = 1;
        gbc.gridy = 5;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.SOUTH;

        JButton botaoConfirma = new JButton( "confirma" );
        botaoConfirma.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                    
                    int valX = x.getValue();
                    int valY = y.getValue();
                    if( valX >= 0 && valX < mapa[0].length &&
                        valY >= 0 && valY < mapa.length       ) {
                        mapa[valY][valX]
                            .alteraCustoDeMovimento( custo.getValue() );
                        String f = ( String )fundo.getSelectedItem();
                        if( !f.equals( "" ) ) {
                            mapa[valY][valX].alteraFundo( f );
                        }
                        else {
                            mapa[valY][valX].alteraFundo( null );
                        }
                    }
                    pcm.centraliza( valX, valY );
                }
            } );
        painelControle.add( botaoConfirma, gbc );

        JPanel painelProximo = new JPanel( new BorderLayout() );

        JButton botaoProximo = new JButton( "Prosseguir" );
        botaoProximo.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                
                    JanelaCriacaoUnidade jcu = new JanelaCriacaoUnidade();
                    dispose();
                    jcu.setSize( 540, 400 );
                    jcu.show();
                }
            } );

        painelProximo.add( botaoProximo, BorderLayout.EAST );
        painelProximo.setBorder( BorderFactory.createEtchedBorder() );

        Container c = getContentPane();

        c.setLayout( new BorderLayout() );

        c.add( pcm, BorderLayout.CENTER );
        c.add( painelControle, BorderLayout.EAST );
        c.add( painelProximo, BorderLayout.SOUTH );

        jfc = new JFileChooser( "arquivos" + File.separator + "mapas" );
    }

    public void inicializaMenu() {
        
        JMenuBar barraMenu =  new JMenuBar();

        JMenu menuMapa = new JMenu( "mapa" );

        JMenuItem novo = new JMenuItem( "novo" );
        novo.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
        
                    JanelaDimensoes jd = 
                        new JanelaDimensoes( JanelaCriacaoMapa.this );
                    Coordenada c = jd.recuperaEscolha();
                    if( c != null ) {                        
                        novoMapa( c.recuperaY(), c.recuperaX() );
                    }
                }
            } );
        
        JMenuItem carregar = new JMenuItem( "carregar" );
        carregar.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    int opcao = jfc.showDialog( JanelaCriacaoMapa.this,
                                                "Carregar" );

                    if (opcao == JFileChooser.APPROVE_OPTION) {
                        File arquivoEntrada = jfc.getSelectedFile();
                        DataInputStream entrada; 
                        try {
                            entrada = 
                                new DataInputStream(
                                    new FileInputStream( arquivoEntrada ) );
                        } catch( FileNotFoundException fnfe ) {
                            return;
                        }
                        try {
                            int linhas = entrada.readInt();
                            int colunas = entrada.readInt();
                            
                            int[][] custos = new int[linhas][colunas];
                            
                            String[][] fundos = new String[linhas][colunas];
                        
                            for( int i = 0; i < linhas; i++ ) {
                                for( int j = 0; j < colunas; j++ ) {
                                    custos[i][j] = entrada.readInt();
                                    String f =  entrada.readUTF();
                                    if( f.equals( "" ) ) {
                                        fundos[i][j] = null;
                                    }
                                    else {
                                        fundos[i][j] = f;
                                    }
                                }
                            }
                            alteraMapa( custos, fundos );
                        } catch( IOException ioe ) {
                            System.err.println( "Erro durante a leitura" );
                        }
                    } 
                }
            } );

        JMenuItem salvar = new JMenuItem( "salvar" );
        salvar.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    int opcao = jfc.showDialog( JanelaCriacaoMapa.this,
                                                "Salvar" );

                    if (opcao == JFileChooser.APPROVE_OPTION) {
                        File arquivoSaida = jfc.getSelectedFile();
                        DataOutputStream saida;
                        try {
                            saida = 
                                new DataOutputStream(
                                    new FileOutputStream( arquivoSaida ) );
                        } catch( FileNotFoundException fnfe ) {
                            return;
                        }                                                
                        
                        int linhas = mapa.length;
                        int colunas = mapa[0].length;
                        try {
                            saida.writeInt( linhas );
                            saida.writeInt( colunas );
                            
                            for( int i = 0; i < linhas; i++ ) {
                                for( int j = 0; j < colunas; j++ ) {
                                    saida.writeInt( 
                                      mapa[i][j].recuperaCustoDeMovimento() );
                                    String f = mapa[i][j].recuperaFundo();
                                    if( f == null ) {
                                        f = "";
                                    }
                                    saida.writeUTF( f );
                                }
                            }
                        } catch( IOException ioe ) {
                            System.err.println( "Erro durante a gravao" );
                        } 
                    }
                }
            } );

        menuMapa.add( novo );
        menuMapa.add( carregar );
        menuMapa.add( salvar );

        barraMenu.add( menuMapa );

        setJMenuBar( barraMenu );
    }

    public void atualiza( Object estado ) {
        
        Coordenada coord;
        try {
            coord = ( Coordenada ) estado;
        } catch( ClassCastException cce ) {
            return;
        }
        
        int valX = coord.recuperaX();
        int valY = coord.recuperaY();
        
        x.setValue( valX );
        y.setValue( valY );
        custo.setValue( mapa[valY][valX].recuperaCustoDeMovimento() );
        String f = mapa[valY][valX].recuperaFundo();
        if( f == null ) {
            fundo.setSelectedItem( "" );
        }
        else {
            fundo.setSelectedItem( f );
        }
    }

    public void novoMapa( int linhas, int colunas ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( linhas, colunas );
        pcm.alteraMapa( mapa );
        ArrayList permitidos = new ArrayList();
        for( int i = 0; i < linhas; i++ ) {
            for( int j = 0; j < colunas; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }                                 
        pcm.alteraPermitidos( permitidos );
        pcm.centraliza( 4, 3 );
    }

    public void alteraMapa( int[][] custos, String[][] fundos ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( custos, fundos );
        ArrayList permitidos = new ArrayList();
        pcm.alteraMapa( mapa );
        for( int i = 0; i < custos.length; i++ ) {
            for( int j = 0; j < custos[0].length; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }                                 
        pcm.alteraPermitidos( permitidos );
        pcm.centraliza( 4, 3 );
    }

    public static void main( String[] args ) {
        
        JFrame jf = new JanelaCriacaoMapa( 15 , 10 );
        jf.pack();
        jf.show();
    }
}
