import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser; 
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

public class JanelaCriacaoUnidade extends JFrame {

    private DefaultListModel modeloUnidades;
    private JList listaUnidades;

    private JFileChooser jfc;

    public JanelaCriacaoUnidade() {
	
	super( "Criacao de Unidades" );
	
	inicializaComponentes();
                
        addWindowListener( new WindowAdapter() {	
		public void windowClosing( WindowEvent we ) {                 
                    System.exit(0);
                }
	    } );
    }

    public void inicializaComponentes() {
	
	modeloUnidades = new DefaultListModel();
	listaUnidades = new JList( modeloUnidades );

        JPanel painelPrincipal = new JPanel( new BorderLayout() );

	JPanel painelBotoes =  new JPanel( new GridLayout( 1, 5 ) );

	JButton botaoNovo = new JButton( "  novo  " );
	botaoNovo.addActionListener( new ActionListener() {
		
		public void actionPerformed( ActionEvent ae ) {
		    
		    JanelaUnidade jUnidade = 
			new JanelaUnidade( JanelaCriacaoUnidade.this );
		    jUnidade.pack();
		    jUnidade.show();
		    
		    Unidade unidade = jUnidade.recuperaUnidade();
		    if( unidade != null ) {
			if( modeloUnidades.contains( unidade ) ) {
                            JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
		                     "No foi possivel criar a unidade.\n" +
                            "J existe uma unidade com o nome especificado", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
                            return;
                            
                        }
                        modeloUnidades.addElement( unidade );
		    }
		}
	    } );
	
	painelBotoes.add( botaoNovo );

	JButton botaoAlterar = new JButton( "alterar" );
	botaoAlterar.addActionListener( new ActionListener() {
		
		public void actionPerformed( ActionEvent ae ) {
		    
		    Unidade unidade = 
			( Unidade ) listaUnidades.getSelectedValue();
		    
		    if( unidade == null ) {
			JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
		               "Por favor, selecione um elemento primeiro.", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
		    }

		    modeloUnidades.removeElement( unidade );
		    
		    JanelaUnidade jUnidade = 
			new JanelaUnidade( JanelaCriacaoUnidade.this ,
					   unidade );
		    jUnidade.pack();
		    jUnidade.show();
		    
		    unidade  = jUnidade.recuperaUnidade();
		    if( unidade != null ) {
			modeloUnidades.addElement( unidade );
		    }
		}
	    } );
        
	painelBotoes.add( botaoAlterar );

	JButton botaoRemover = new JButton( "remover" );
	botaoRemover.addActionListener( new ActionListener() {
		
		public void actionPerformed( ActionEvent ae ) {
		    
		    Unidade unidade = 
			( Unidade ) listaUnidades.getSelectedValue();
		    
		    if( unidade == null ) {
			JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
		               "Por favor, selecione um elemento primeiro.", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
		    }

		    modeloUnidades.removeElement( unidade );
		}
	    } );

	painelBotoes.add( botaoRemover );

        JButton botaoCarregar = new JButton( "carregar" );
        botaoCarregar.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    jfc.setMultiSelectionEnabled( true ); 
                    int opcao = 
                        jfc.showDialog( JanelaCriacaoUnidade.this,
                                        "Carregar");

                    if (opcao == JFileChooser.APPROVE_OPTION) {
                        File[] arquivosEntrada = jfc.getSelectedFiles();
                        
                        for( int i = 0; i < arquivosEntrada.length; i ++ ) {
                            DataInputStream entrada; 
                            try {
                                entrada = 
                                  new DataInputStream(
                                   new FileInputStream( arquivosEntrada[i] ) );
                            } catch( FileNotFoundException fnfe ) {
                                return;
                            }
                            try {
                                String nome = entrada.readUTF();
                                int ptsDeVida = entrada.readInt();
                                int precisao = entrada.readInt();
                                int ptsDeMovimento = entrada.readInt();
                                String figura = entrada.readUTF();
                                if( figura == "" ) {
                                    figura = null;
                                }
                                Unidade unidade = new Unidade( nome,
                                                               ptsDeVida,
                                                               precisao,
                                                               ptsDeMovimento,
                                                               figura );
                                int numAcoes = entrada.readInt();
                                
                                Gerenciador g = 
                                    Gerenciador.recuperaInstancia();
                                for( int j = 0; j < numAcoes; j++ ) {
                                    String nomeDaAcao = entrada.readUTF();
                                    AcaoAbstrata acao = 
                                        g.recuperaAcao( nomeDaAcao );
                                    unidade.adicionaAcao( acao );
                                }
                                
                                if( modeloUnidades.contains( unidade ) ) {
                                    JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
		                     "No foi possivel criar a unidade de "+
                                      arquivosEntrada[i].getName() + ".\n" +
                                  "J existe uma unidade com um mesmo nome", 
                                         "Erro", JOptionPane.ERROR_MESSAGE);
                                    return;
                                }
                                modeloUnidades.addElement( unidade );
                            } catch( IOException ioe ) {
                                System.err.println( "Erro durante a leitura" );
                            }
                        }
                    }
                    jfc.setMultiSelectionEnabled( false ); 
                }
            } );        

	painelBotoes.add( botaoCarregar );

	JButton botaoSalvar = new JButton( " salvar " );
        botaoSalvar.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent ae ) {

                    Unidade unidade = 
			( Unidade ) listaUnidades.getSelectedValue();
		    
		    if( unidade == null ) {
			JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
		               "Por favor, selecione um elemento primeiro.", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
		    }

                    int opcao = 
                        jfc.showDialog( JanelaCriacaoUnidade.this,
                                        "Salvar" );
                    
                    if (opcao == JFileChooser.APPROVE_OPTION) {
                        File arquivoSaida = jfc.getSelectedFile();
                        DataOutputStream saida;
                        try {
                            saida = 
                                new DataOutputStream(
                                    new FileOutputStream( arquivoSaida ) );
                        } catch( FileNotFoundException fnfe ) {
                            return;
                        }                                                
                        
                        
                        try {
                            saida.writeUTF( unidade.recuperaNome() );
                            saida.writeInt( unidade.recuperaPontosDeVida() );
                            saida.writeInt( unidade.recuperaPrecisao() );
                            saida.writeInt( 
                                  unidade.recuperaPontosDeMovimento() );
                            String f = unidade.recuperaFigura();
                            if( f == null ) {
                                f = "";
                            }
                            saida.writeUTF( f );
                            
                            AcaoAbstrata[] acoes = unidade.recuperaAcoes();
                            saida.writeInt( acoes.length );
                            
                            for( int i = 0; i < acoes.length; i++ ) {
                                saida.writeUTF( acoes[i].recuperaNome() );
                            }
                            
                        } catch( IOException ioe ) {
                            System.err.println( "Erro durante a gravao" );
                        } 
                    }
                }
            } );

	painelBotoes.add( botaoSalvar );

	listaUnidades.setSelectionMode( 
                     DefaultListSelectionModel.SINGLE_SELECTION );

        jfc = new JFileChooser( "arquivos" + File.separator + "unidades" );

        painelPrincipal.add( 
                  new JScrollPane( listaUnidades ), BorderLayout.CENTER );
        painelPrincipal.add( painelBotoes, BorderLayout.SOUTH );
        painelPrincipal.setBorder( 
              BorderFactory.createCompoundBorder(
                    BorderFactory.createTitledBorder( "Unidades" ),
                    BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );

        JPanel painelProximo = new JPanel( new BorderLayout() );

        JButton botaoProximo = new JButton( "prosseguir" );
        botaoProximo.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {

                    if( modeloUnidades.size() == 0 ) {
                        JOptionPane.showMessageDialog( 
                                                  JanelaCriacaoUnidade.this,
                   "Ateno,  preciso especificar pelo menos um prototipo", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                    Gerenciador gerenciador = 
                        Gerenciador.recuperaInstancia();
                    for( int i = 0; i < modeloUnidades.size(); i++ ) {
                        Unidade unidade = 
                            ( Unidade ) modeloUnidades.getElementAt( i );
                        gerenciador.criaPrototipo( unidade );
                    }
                    JanelaNumeroUnidades proximaJanela = 
                        new JanelaNumeroUnidades();
                    dispose();
                    proximaJanela.setSize( 540, 400 );
                    proximaJanela.show();
                }
            } );        
        
        painelProximo.add( botaoProximo, BorderLayout.EAST );
        
        JComponent c = ( JComponent )getContentPane();

	c.setLayout( new BorderLayout() );
        
        c.add( painelPrincipal, BorderLayout.CENTER );
        c.add( painelProximo, BorderLayout.SOUTH );
    }
    
    public static void main( String[] args ) {
	
	JFrame jf = new JanelaCriacaoUnidade();
	jf.pack();
	jf.show();
    }
}
