import java.util.ArrayList;
import java.util.Random;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

public class JanelaEscolhaPosicoes extends JFrame implements Observador {

    static final int NENHUM = -1;
    static final int POSICIONA = 0;
    static final int REMOVE = 1;

    private PainelControleMapa pcm;
    private Setor[][] mapa;

    private JLabel nomeJogador;
    private DefaultListModel modeloNaoPosicionadas;
    private JList unidadesNaoPosicionadas;

    private Jogador[] jogadores;
    private int jogadorAtual;

    private ArrayList coordenadasInclusao;
    private ArrayList coordenadasRemocao;
    
    private Random rand;

    private int escolha;

    private Unidade inicial;

    public JanelaEscolhaPosicoes() {

        super( "Escolha as posies de suas unidades" );
        
        inicializaComponentes();
        
        addWindowListener( new WindowAdapter() {	
		public void windowClosing( WindowEvent we ) {                 
                    System.exit(0);
                }
	    } );

        rand = new Random();
        
        Gerenciador gerenciador = Gerenciador.recuperaInstancia();
        jogadores = gerenciador.recuperaJogadores();
        jogadorAtual = 0;
               
        proximoJogador();
    }

    public void inicializaComponentes() {

        nomeJogador = new JLabel( " " );

        Gerenciador gerenciador = Gerenciador.recuperaInstancia();

        mapa = gerenciador.recuperaMapa();
        pcm = new PainelControleMapa( mapa );
        pcm.adicionaObservador( this );

        JPanel painelControle = new JPanel( new BorderLayout() );
        painelControle.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        painelControle.setPreferredSize( new Dimension( 150, 350 ) );

        modeloNaoPosicionadas = new DefaultListModel();
        unidadesNaoPosicionadas = new JList( modeloNaoPosicionadas );

        unidadesNaoPosicionadas.setSelectionMode( 
                             DefaultListSelectionModel.SINGLE_SELECTION );

        JPanel painelBotoes = new JPanel( new GridLayout( 2, 1 ) );
        
        JButton botaoIncluir = new JButton( "Incluir" );
        botaoIncluir.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                
                    escolha = JanelaEscolhaPosicoes.POSICIONA;
                    pcm.alteraPermitidos( coordenadasInclusao );
                }
            } );
        
        JButton botaoRemover = new JButton( "Remover" );
        botaoRemover.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {

                    escolha = JanelaEscolhaPosicoes.REMOVE;
                    pcm.alteraPermitidos( coordenadasRemocao );
                }
            } );
        
        painelBotoes.add( botaoIncluir );
        painelBotoes.add( botaoRemover );

        painelControle.add( new JLabel( "Unidades:"), BorderLayout.NORTH );  
        painelControle.add( new JScrollPane( unidadesNaoPosicionadas ),
                            BorderLayout.CENTER );
        painelControle.add( painelBotoes, BorderLayout.SOUTH );

        JPanel painelProximo = new JPanel( new BorderLayout() );
        
        JButton botaoProximo = new JButton( "Prosseguir" );
        botaoProximo.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                    
		    if( modeloNaoPosicionadas.size() != 0 ) {
			JOptionPane.showMessageDialog( 
			      JanelaEscolhaPosicoes.this,
			      "Ateno, todas as unidades devem ser " + 
			      "posicionadas.", 
                              "Erro", JOptionPane.ERROR_MESSAGE);
			return;
		    }


                    Gerenciador gerenciador = 
                        Gerenciador.recuperaInstancia();
                    
                    if( ++jogadorAtual < 
                        gerenciador.recuperaTotalJogadores() ) {
                        
                        proximoJogador();
                        return;
                    }
                    JanelaTurno proximaJanela = 
                        new JanelaTurno();
		    Gerenciador.recuperaInstancia().adicionaObservador( 
							       proximaJanela );
                    dispose();
                    proximaJanela.setSize( 540, 400 );
                    proximaJanela.show();
                }
            } );

        painelProximo.add( botaoProximo, BorderLayout.EAST );

        painelProximo.setBorder( BorderFactory.createEtchedBorder() );

        Container c = getContentPane();
        
        c.setLayout( new BorderLayout() );
        c.add( nomeJogador, BorderLayout.NORTH );
        c.add( pcm, BorderLayout.CENTER );
        c.add( painelControle, BorderLayout.EAST );
        c.add( painelProximo, BorderLayout.SOUTH );
    }

    public void atualiza( Object estado ) {
        
        Coordenada coord;
        try {
            coord = ( Coordenada ) estado;
        } catch( ClassCastException cce ) {
            return;
        }
        
        switch( escolha ) {
        case JanelaEscolhaPosicoes.POSICIONA:
            Unidade unidade = 
                ( Unidade ) unidadesNaoPosicionadas.getSelectedValue();
		    
            if( unidade == null ) {
                JOptionPane.showMessageDialog( this,
                                "Por favor, selecione uma unidade primeiro.", 
                                "Erro", JOptionPane.ERROR_MESSAGE);
                return;
            }

            posicionaUnidade( unidade, coord );
            escolha = JanelaEscolhaPosicoes.NENHUM;
            pcm.alteraPermitidos( new ArrayList() );
            break;
        case JanelaEscolhaPosicoes.REMOVE:
            removeUnidade( coord );
            escolha = JanelaEscolhaPosicoes.NENHUM;
            pcm.alteraPermitidos( new ArrayList() );
            break;
        default:
            break;
        }
        
    }
    
    private void proximoJogador() {

        escolha = JanelaEscolhaPosicoes.NENHUM;

        nomeJogador.setText( jogadores[ jogadorAtual ].recuperaNome() );
        
        modeloNaoPosicionadas.removeAllElements();
        Unidade[] unidades = jogadores[ jogadorAtual ].recuperaUnidades();
        
        for( int i = 0; i < unidades.length; i++ ) {
            modeloNaoPosicionadas.addElement( unidades[i] );
        }
        
        coordenadasInclusao = new ArrayList();
        coordenadasRemocao = new ArrayList();

        int x, y;
        do {
            x = rand.nextInt( mapa[0].length );
            y = rand.nextInt( mapa.length );
        } while( ! mapa[y][x].estaVazio() );

        Coordenada coord = new Coordenada( x, y );
        posicionaUnidade( unidades[0], coord );
        inicial = unidades[0];
        pcm.centraliza( x, y );
    }   

    private void posicionaUnidade( Unidade unidade, Coordenada coordenada ) {

        int x = coordenada.recuperaX();
        int y = coordenada.recuperaY();

        unidade.move( mapa[y][x] );
        mapa[y][x].alteraUnidade( unidade );

        coordenadasRemocao.add( coordenada );
        adicionaCoordenadas( x, y );
        
        modeloNaoPosicionadas.removeElement( unidade );
        pcm.repaint();
    }

    private void removeUnidade( Coordenada coordenada ) {
        
        int x = coordenada.recuperaX();
        int y = coordenada.recuperaY();

        Unidade unidade = mapa[y][x].recuperaUnidade();
        
        if( unidade == inicial ) {
            
            JOptionPane.showMessageDialog( this,
                         "Ateno, voc no pode remover a unidade inicial.", 
                                "Erro", JOptionPane.ERROR_MESSAGE);
                return;
        }
        
        mapa[y][x].removeUnidade();

        int idx = coordenadasRemocao.indexOf( coordenada );
        coordenadasRemocao.remove( idx );
        coordenadasInclusao = new ArrayList();

        for( int i = 0; i < coordenadasRemocao.size(); i++ ) {
            Coordenada coord = ( Coordenada ) coordenadasRemocao.get( i );
            adicionaCoordenadas( coord.recuperaX(), coord.recuperaY() );
        }
        modeloNaoPosicionadas.addElement( unidade );
    }

    private void adicionaCoordenadas( int x, int y ) {
        
        for( int i = -2; i <= 2; i++ ) {
            if( y + i >= 0 && y + i < mapa.length ) {
                for( int j = -2; j <= 2; j++ ) {
                    
                    if( x + j >= 0 && x + j < mapa[0].length ) {
                        if( mapa[y + i][x + j].estaVazio() ) {
                            Coordenada coord = new Coordenada( x + j , y + i );
                            if( ! coordenadasInclusao.contains( coord ) ) {
                                coordenadasInclusao.add( coord );
                            }
                        }
                    }
                }
            } 
        }
    }

    public static void main( String[] args ) {

        Gerenciador.recuperaInstancia().geraMapa();
        
        JFrame jf = new JanelaEscolhaPosicoes();
        jf.pack();
        jf.show();
    }
}
