import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;


/**
 *
 * @version 1.0
 * @author Alex Minoru Kusano & Marcos Roberto Yukio Koga
 */
public class JanelaNumeroUnidades extends JFrame {

    /** Nmero nico de unidades para todos os jogadores. */
    private int totalUnidades;

    /** Vetor com o nmero de unidades de cada jogador. */
    private int[] totalUnidadesJogador;

    /** Lista de jogadores. */
    private JComboBox cmbJogadores;

    /** Campo de texto para o nmero de unidades. */
    private CampoNumeroInteiro txtNumero;

    /** Checkbox para indicar se o nmero de unidades  comum para todos os
	jogadores. */
    private JCheckBox checkTodos;

    /** Boto para confirmar o nmero de unidades. */
    private JButton botaoOK;

    /** Boto para ir para a prxima tela. */
    private JButton botaoProsseguir;


    public JanelaNumeroUnidades() {

	super( "Nmero de unidades" );
	inicializaComponentes();
        addWindowListener( new WindowAdapter() {	
		public void windowClosing( WindowEvent we ) {                 
                    System.exit(0);
                }
	    } );
    }

    
    private void inicializaComponentes() {

	int totalJogadores;

	// Obtm o total de jogadores
	totalJogadores = 
	    Gerenciador.recuperaInstancia().recuperaTotalJogadores();

	// Inicializa vetor com total de unidades de cada jogador
	totalUnidadesJogador = new int[ totalJogadores ];
	for( int i = 0; i < totalJogadores; i++ ) {
	    totalUnidadesJogador[i] = 1;
	}

	// Inicializa total de unidades para todos os jogadores
	totalUnidades = 0;

	// Inicializa a lista de jogadores
	String[] jogadores = new String[ totalJogadores ];
	for( int i = 0; i < totalJogadores; i++ ) {
	    jogadores[i] = new String( "Jogador " + (i + 1) );
	}
	cmbJogadores = new JComboBox( jogadores );
	cmbJogadores.addItemListener( new JogadorListener() );
	cmbJogadores.setSelectedIndex( -1 );
	cmbJogadores.setEnabled( false );

	txtNumero = new CampoNumeroInteiro( 1, 5 );
	txtNumero.setHorizontalAlignment( JTextField.RIGHT );
	
	botaoOK = new JButton( "OK" );
	botaoOK.addActionListener( new OK() );

	JPanel painelUnidades = new JPanel( new FlowLayout() );
	painelUnidades.add( new JLabel( "Nmero de unidades:" ) );
	painelUnidades.add( txtNumero );
	painelUnidades.add( botaoOK );
	painelUnidades.setAlignmentX( Component.LEFT_ALIGNMENT );

	checkTodos = new JCheckBox( "Mesmo nmero de unidades para todos" );
	checkTodos.setAlignmentX( Component.LEFT_ALIGNMENT );
	checkTodos.setSelected( true );
	checkTodos.addItemListener( new Todos() );

	Box painelCentral = Box.createVerticalBox();
	painelCentral.add( cmbJogadores );
	painelCentral.add( painelUnidades );
	painelCentral.add( checkTodos );
	painelCentral.add( Box.createVerticalStrut( 30 ) );

	botaoProsseguir = new JButton( "Prosseguir" );
        botaoProsseguir.setToolTipText( "clique aqui para continuar" );
	botaoProsseguir.addActionListener( new Prosseguir() );

	JPanel painelSul = new JPanel( new BorderLayout() );
	painelSul.add( botaoProsseguir, BorderLayout.EAST );
	painelSul.setBorder( BorderFactory.createEmptyBorder( 3, 0, 0, 0 ) );

	Container container = this.getContentPane();
	container.setLayout( new BorderLayout() );
	container.add( new JScrollPane( painelCentral ), BorderLayout.CENTER );
	container.add( painelSul, BorderLayout.SOUTH );
    }
    

    private class JogadorListener implements ItemListener {

	public void itemStateChanged( ItemEvent ie ) {

	    int jogador = cmbJogadores.getSelectedIndex();
	    if( jogador >= 0 ) {
		txtNumero.setText( 
			  Integer.toString( totalUnidadesJogador[jogador] ) );
	    }
	}
    }


    private class Todos implements ItemListener {

	public void itemStateChanged( ItemEvent ie ) {

	    // Desativa ou ativa a lista de jogadores
	    if( checkTodos.isSelected() ) {
		cmbJogadores.setEnabled( false );
		cmbJogadores.setSelectedIndex( -1 );
	    } else {
		cmbJogadores.setEnabled( true );
	    }
	}
    }


    private class OK implements ActionListener {

	public void actionPerformed( ActionEvent ae ){

	    String numeroDigitado = txtNumero.getText().trim();
	    int numero;

	    // Verifica se um jogador foi selecionado
	    if( !checkTodos.isSelected() && 
		cmbJogadores.getSelectedIndex() == -1  ) {
		Toolkit.getDefaultToolkit().beep();
		cmbJogadores.requestFocus();

		JOptionPane.showMessageDialog( JanelaNumeroUnidades.this, 
		       "Por favor, selecione um jogador.", "Erro",
			JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Verifica se o campo nmero de unidades est vazio
	    if( numeroDigitado.equals( "" ) ) {
		Toolkit.getDefaultToolkit().beep();
		txtNumero.requestFocus();

		// Consertar frame principal...
		JOptionPane.showMessageDialog( JanelaNumeroUnidades.this, 
		       "Por favor, digite o nmero de unidades.", "Erro",
			JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Verifica se o nmero de unidades  vlido
	    numero = Integer.parseInt( numeroDigitado );
	    if( numero < 1 ) {
		Toolkit.getDefaultToolkit().beep();
		txtNumero.requestFocus();

		// Consertar frame principal...
		JOptionPane.showMessageDialog( JanelaNumeroUnidades.this, 
		       "O nmero mnimo de unidades  um.", "Erro",
			JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Verifica se deve atualizar de todos ou de apenas um jogador
	    if( checkTodos.isSelected() ) {
		// Atualiza o nmero de unidades de todos
		for( int i = 0; i < totalUnidadesJogador.length; i++ ) {
		    totalUnidadesJogador[i] = numero;
		}
	    } else {
		// Atualiza o nmero de unidades do jogador
		int jogador = cmbJogadores.getSelectedIndex();
		totalUnidadesJogador[jogador] = numero;
	    }
	}
    }


    private class Prosseguir implements ActionListener {

	public void actionPerformed( ActionEvent ae ){

	    // Atualiza no gerenciador o nmero de unidades de cada jogador
	    Gerenciador gerenciador = Gerenciador.recuperaInstancia();

	    for( int i = 0; i < totalUnidadesJogador.length; i++ ) {
		gerenciador.alteraTotalDeUnidades( i,
			   totalUnidadesJogador[i] );
		}

	    JanelaEscolhaUnidades proximaJanela = 
                new JanelaEscolhaUnidades();
	    dispose();
            proximaJanela.setSize( 540, 400 );
	    proximaJanela.show();
	}
    }

    public static void main( String[] args ) {
        
	Gerenciador gerenciador = Gerenciador.recuperaInstancia();
	gerenciador.novoJogo( 4 );
        JFrame janela = new JanelaNumeroUnidades();
	//        janela.pack();
	janela.setSize(540, 400);
        janela.show();
	System.out.println( janela.getSize() );
    }
}






