import java.util.ArrayList;
import java.util.Collection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.DefaultListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


public class JanelaTurno extends JFrame implements Observador {

    static final int NENHUM = -1;
    static final int MOVER = 0;
    static final int AGIR = 1;

    private int opcao;
    
    private DefaultListModel modeloDaLista;

    private JList listaUnidades;

    private Setor[][] mapa;

    private PainelControleMapa pcm;

    private ArrayList unidadesJaMoveram;

    private ArrayList unidadesJaAgiram;

    private AcaoAbstrata acaoEscolhida;

    private Jogador[] jogadores;

    private int jogadorAtual;

    private JLabel nomeJogador;

    public JanelaTurno() {
        
        super( "Execute seus movimentos/aes" );
        inicializaComponentes();
        addWindowListener( new WindowAdapter() {	
		public void windowClosing( WindowEvent we ) {                 
                    System.exit(0);
                }
	    } );
	unidadesJaMoveram = new ArrayList();
	unidadesJaAgiram = new ArrayList();
    }


    private void inicializaComponentes() {

	Gerenciador gerenciador = Gerenciador.recuperaInstancia();        

        JPanel painelControle = new JPanel( new GridBagLayout() );
        painelControle.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        painelControle.setPreferredSize( new Dimension ( 150, 350 ) );

	// Adiciona label unidades
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridx = 0;
        gbc.weightx = 1;
        gbc.gridy = 0;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.NORTH;
        painelControle.add( new JLabel( "Unidades:"), gbc );

	// Adiciona lista de unidades do jogador
	modeloDaLista = new DefaultListModel();
	listaUnidades = new JList ( modeloDaLista );
	jogadorAtual = gerenciador.proximoJogador();
	Unidade[] unidades = gerenciador.recuperaUnidades( jogadorAtual );
	for( int i = 0; i < unidades.length; i++ ) {
	    modeloDaLista.addElement( unidades[i] );
	}
	listaUnidades.setSelectedIndex(-1);
	listaUnidades.setSelectionMode( 
		      DefaultListSelectionModel.SINGLE_SELECTION );
	listaUnidades.addListSelectionListener( new SelecionarUnidade() );
        gbc.gridy = 1;
        painelControle.add( new JScrollPane( listaUnidades ), gbc );

	// Adiciona boto Mover
	JButton botaoMover = new JButton( "Mover" );
	botaoMover.addActionListener( new Mover() );
        gbc.gridy = 2;
        painelControle.add( botaoMover, gbc );

	// Adiciona boto Agir
	JButton botaoAgir = new JButton( "Agir" );
	botaoAgir.addActionListener( new Agir() );
        gbc.gridy = 3;
        painelControle.add( botaoAgir, gbc );


	// Recupera o mapa do jogo
        mapa = gerenciador.recuperaMapa();      
        pcm = new PainelControleMapa( mapa );
        pcm.setBorder( BorderFactory.createEtchedBorder() );

	// Adiciona o objeto JanelaTurno na lista de observadores do mapa
        pcm.adicionaObservador( this );


        JPanel painelProximo = new JPanel( new BorderLayout() );

        JButton botaoProximo = new JButton( "Terminar turno" );
        botaoProximo.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                
		    Gerenciador gerenciador = Gerenciador.recuperaInstancia();
		    int jogadorAtual = gerenciador.proximoJogador();
		    Unidade[] unidades = 
			gerenciador.recuperaUnidades( jogadorAtual );
		    modeloDaLista.clear();
		    for( int i = 0; i < unidades.length; i++ ) {
			modeloDaLista.addElement( unidades[i] );
		    }
		    listaUnidades.setSelectedIndex(-1);		    
		    unidadesJaMoveram.clear();
		    unidadesJaAgiram.clear();
                    nomeJogador.setText( 
				jogadores[ jogadorAtual ].recuperaNome() );
		    opcao = JanelaTurno.NENHUM;
		    pcm.alteraPermitidos( new ArrayList() );
                }
            } );

        painelProximo.add( botaoProximo, BorderLayout.EAST );
        painelProximo.setBorder( BorderFactory.createEtchedBorder() );
        

        // Cria rtulo com nome do jogador
        jogadores = gerenciador.recuperaJogadores();
        nomeJogador = new JLabel( jogadores[ jogadorAtual ].recuperaNome() );
        
        Container c = getContentPane();

        c.setLayout( new BorderLayout() );

        c.add( nomeJogador, BorderLayout.NORTH );
        c.add( pcm, BorderLayout.CENTER );
        c.add( painelControle, BorderLayout.EAST );
        c.add( painelProximo, BorderLayout.SOUTH );
    }


    public void atualiza( Object estado ) {

	Coordenada coord;
        try {
            coord = ( Coordenada ) estado;
	    

        } catch( ClassCastException cce ) {

	    try {
		Unidade unidade = (Unidade) estado;

		int indice = modeloDaLista.indexOf( unidade );

		if( indice != -1 ) {
		    modeloDaLista.removeElementAt( indice );
		    pcm.repaint();
		}
		
	    }  catch( ClassCastException c ) {  }

	    return;
        }
         
        int valX = coord.recuperaX();
        int valY = coord.recuperaY();

	int indice = listaUnidades.getSelectedIndex();
	Unidade unidade = (Unidade) modeloDaLista.getElementAt( indice );

	// Mover
        switch( opcao ) {
        case JanelaTurno.MOVER:
            Gerenciador.recuperaInstancia().moveUnidade( unidade, valX, valY );
	    unidadesJaMoveram.add( unidade );
	    pcm.alteraPermitidos( new ArrayList() );
	    opcao = JanelaTurno.NENHUM;
            break;
        case JanelaTurno.AGIR:
            
            Gerenciador.recuperaInstancia().executaAcao( unidade,
                                                         acaoEscolhida,
                                                         valX,
                                                         valY );
            unidadesJaAgiram.add( unidade );
	    pcm.alteraPermitidos( new ArrayList() );
	    opcao = JanelaTurno.NENHUM;
            break;
        default:
            break;
        }
        pcm.centraliza( valX, valY );
    }

  
    private class SelecionarUnidade implements ListSelectionListener {

	public void valueChanged( ListSelectionEvent e ) {

	    // Centraliza o mapa na unidade selecionada
	    int indice = listaUnidades.getSelectedIndex();

	    if( indice < 0 ) {

		return;
	    }
	    Unidade unidade = (Unidade) modeloDaLista.getElementAt( indice );
	    int x = unidade.recuperaPosicao().recuperaX();
	    int y = unidade.recuperaPosicao().recuperaY();
            if( opcao != JanelaTurno.NENHUM ) {
                pcm.alteraPermitidos( new ArrayList() );
                pcm.alteraAfetados( new ArrayList() );
                opcao = JanelaTurno.NENHUM;
            }
	    pcm.centraliza( x, y );
	}
    }


    private class Mover implements ActionListener {

	public void actionPerformed(ActionEvent ae) {

            Unidade unidade = 
                ( Unidade ) listaUnidades.getSelectedValue();
            
            if( unidade == null ) {
                JOptionPane.showMessageDialog( JanelaTurno.this,
                        "Por favor, selecione um elemento primeiro.", 
                        "Erro", JOptionPane.ERROR_MESSAGE);
                return;
            }

	    for( int i = 0 ; i < unidadesJaMoveram.size(); i++ ) {
		if( unidade == ( Unidade ) unidadesJaMoveram.get( i ) ) {
		    
		    JOptionPane.showMessageDialog( JanelaTurno.this,
			"Ateno, a unidade escolhida j moveu nesse turno",
			"Erro", JOptionPane.ERROR_MESSAGE );

		    return;
		}
	    }

	    Collection permitidos = 
		Gerenciador.recuperaInstancia().recuperaPosicoes( unidade );
	    
	    pcm.alteraPermitidos( permitidos );
            pcm.alteraAfetados( new ArrayList() );
            
	    opcao = JanelaTurno.MOVER;
	}
    }

    private class Agir implements ActionListener {

	public void actionPerformed(ActionEvent ae) {
            
            Unidade unidade = 
                ( Unidade ) listaUnidades.getSelectedValue();
            
            if( unidade == null ) {
                JOptionPane.showMessageDialog( JanelaTurno.this,
                        "Por favor, selecione um elemento primeiro.", 
                        "Erro", JOptionPane.ERROR_MESSAGE);
                return;
            }

            for( int i = 0 ; i < unidadesJaAgiram.size(); i++ ) {
		if( unidade == ( Unidade ) unidadesJaAgiram.get( i ) ) {
		    
		    JOptionPane.showMessageDialog( JanelaTurno.this,
			"Ateno, a unidade escolhida j agiu nesse turno",
			"Erro", JOptionPane.ERROR_MESSAGE );

		    return;
		}
	    }
            
            Box painelAcoes = Box.createVerticalBox();
            JList listaAcoes = new JList( unidade.recuperaAcoes() );
            listaAcoes.setSelectionMode( 
                    DefaultListSelectionModel.SINGLE_SELECTION );
            
            painelAcoes.add( new JLabel( "Escolha a ao a ser executada:" ) );
            painelAcoes.add( new JScrollPane( listaAcoes ) );
           
            int escolha = 
                JOptionPane.showConfirmDialog( JanelaTurno.this,
                                               painelAcoes, "Escolha uma ao",
                                               JOptionPane.OK_CANCEL_OPTION,
                                               JOptionPane.PLAIN_MESSAGE );
            
            if( escolha == JOptionPane.OK_OPTION ) {
                acaoEscolhida = 
                    ( AcaoAbstrata )listaAcoes.getSelectedValue();
                
                if( acaoEscolhida == null ) {
                    JOptionPane.showMessageDialog( JanelaTurno.this,
                       "Ateno, voc tem de escolher uma ao a ser tomada.", 
                       "Erro", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                
                Collection permitidos = 
		Gerenciador.recuperaInstancia().recuperaPosicoes( unidade,
                                                             acaoEscolhida );

                Collection afetados = acaoEscolhida.calculaArea();

                pcm.alteraPermitidos( permitidos );
                pcm.alteraAfetados( afetados );
                
                opcao = JanelaTurno.AGIR;
            }
        }
    }


    public static void main( String[] args ) {
        
        JFrame jt = new JanelaNumeroJogadores();
        jt.pack();
        jt.show();
    }
}













