import java.io.File;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

public class JanelaUnidade extends JDialog {

    private JTextField nome;
    private CampoNumeroInteiro pontosDeVida;
    private CampoNumeroInteiro precisao;
    private CampoNumeroInteiro pontosDeMovimento;

    private JComboBox figura;
    
    private Unidade unidade;
    
    private DefaultListModel modeloDisponiveis;
    private JList acoesDisponiveis;
    private DefaultListModel modeloEscolhidos;
    private JList acoesEscolhidas;

    public JanelaUnidade( JFrame dono ) {

        super( dono, "Criar nova Unidade", true );
        inicializaComponentes();
    }

    public JanelaUnidade( JFrame dono, Unidade unidade ) {
        
        super( dono, "Alterar Unidade", true );
        
        this.unidade = unidade;
        inicializaComponentes( unidade );
    }

    public void inicializaComponentes() {

        JPanel painelAtributos = new JPanel( new GridLayout( 5, 2 ) );

        nome = new JTextField( 5 );
        pontosDeVida = new CampoNumeroInteiro( 5 );
        precisao = new CampoNumeroInteiro( 5 );
        pontosDeMovimento = new CampoNumeroInteiro( 5 );
        
        File imagens = new File( "imagens" + File.separator + "unidades" );
        if( imagens.isDirectory() ) {
            figura = new JComboBox( imagens.list() );
        }
        else {
            figura = new JComboBox();
        }
        figura.addItem( "" );
        figura.setSelectedItem( "" );

        painelAtributos.add( new JLabel( "Nome:" ) );
        painelAtributos.add( nome );

        painelAtributos.add( new JLabel( "Pts de Vida:" ) );
        painelAtributos.add( pontosDeVida );

        painelAtributos.add( new JLabel( "Preciso:" ) );
        painelAtributos.add( precisao );

        painelAtributos.add( new JLabel( "Pts de Movimento:" ) );
        painelAtributos.add( pontosDeMovimento );

        painelAtributos.add( new JLabel( "Figura:" ) );
        painelAtributos.add( figura );

        painelAtributos.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        JPanel painelAcoes = new JPanel( new GridLayout( 1, 3 ) );

        modeloDisponiveis = new DefaultListModel();
        acoesDisponiveis = new JList( modeloDisponiveis );

        painelAcoes.add( new JScrollPane( acoesDisponiveis ) );

        AcaoAbstrata[] acoes = 
            Gerenciador.recuperaInstancia().recuperaAcoes();
        
        for( int i = 0; i< acoes.length; i++ ) {
            modeloDisponiveis.addElement( acoes[i] );
        }

        JPanel painelBotoes = new JPanel( new GridBagLayout() );
        
        GridBagConstraints gbc = new GridBagConstraints();     

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.gridx = 0;
        gbc.gridy = 0;

        JButton botaoIncluir = new JButton( "incluir" );

        botaoIncluir.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    Object[] acoes = acoesDisponiveis.getSelectedValues();
                    for( int i = 0; i < acoes.length; i++ ) {
                        modeloEscolhidos.addElement( acoes[i] );
                        modeloDisponiveis.removeElement( acoes[i] );
                    }
                }
            } ); 
        
        painelBotoes.add( botaoIncluir, gbc );

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.gridx = 0;
        gbc.gridy = 1;

        JButton botaoExcluir = new JButton( "excluir" );
        botaoExcluir.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    Object[] acoes = acoesEscolhidas.getSelectedValues();
                    for( int i = 0; i < acoes.length; i++ ) {
                        modeloDisponiveis.addElement( acoes[i] );
                        modeloEscolhidos.removeElement( acoes[i] );
                    }
                }
            } ); 
        painelBotoes.add( botaoExcluir, gbc );

        painelBotoes.setBorder( 
                      BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );

        painelAcoes.add( painelBotoes );

        modeloEscolhidos = new DefaultListModel();
        acoesEscolhidas = new JList( modeloEscolhidos );

        painelAcoes.add( new JScrollPane( acoesEscolhidas ) );

        painelAcoes.setBorder( 
              BorderFactory.createCompoundBorder( 
                    BorderFactory.createTitledBorder( "Aes" ),
                    BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        
        JPanel painelConfirma = 
            new JPanel( new FlowLayout( FlowLayout.RIGHT ) );
        
        JButton botaoCancelar = new JButton( " cancelar " );
        botaoCancelar.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    hide();
                }
            } );
        
        JButton botaoConfirmar = new JButton( "confirmar" );
        botaoConfirmar.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    
                    if( nome.getText().trim().equals( "" ) ) {
                        JOptionPane.showMessageDialog( 
                                                  JanelaUnidade.this,
		                "Por favor, entre com um nome para a unidade", 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
                    }
                    
                    if( pontosDeVida.getValue() <= 0 ) {
                        JOptionPane.showMessageDialog( 
                                                        JanelaUnidade.this,
		                "Por favor, entre com um valor positivo " +
                                                 "para os pontos de vida" , 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
                    }

                    if( precisao.getValue() <= 0 ) {
                        JOptionPane.showMessageDialog( 
                                                        JanelaUnidade.this,
		                "Por favor, entre com um valor positivo " +
                                                        "para a precisao" , 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
                    }

                    if( pontosDeMovimento.getValue() <= 0 ) {
                        JOptionPane.showMessageDialog( 
                                                        JanelaUnidade.this,
		                "Por favor, entre com um valor positivo " +
                                            "para os pontos de Movimento" , 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
                    }

                    if( modeloEscolhidos.size() == 0 ) {
                        
                        JOptionPane.showMessageDialog( 
                                                        JanelaUnidade.this,
		                      "Por favor, inclua pelo menos uma " +
                                                    "acao para a unidade" , 
				           "Erro", JOptionPane.ERROR_MESSAGE);
			return;
                    }
                    
                    
                    if( unidade == null ) {
                        String f = ( String )figura.getSelectedItem();
                        if( f.equals( "" ) ){
                            f = null;
                        }

                        unidade = new Unidade( nome.getText().trim(), 
                                               pontosDeVida.getValue(), 
                                               precisao.getValue(), 
                                               pontosDeMovimento.getValue(),
                                               f );
                        for( int i = 0; i < modeloDisponiveis.size(); i++ ) {
                            unidade.removeAcao( 
                                ( AcaoAbstrata )modeloDisponiveis.get( i ) );
                        }
                        for( int i = 0; i < modeloEscolhidos.size(); i++ ) {
                            unidade.adicionaAcao( 
                                ( AcaoAbstrata )modeloEscolhidos.get( i ) );
                        }
                    }
		    else {
			unidade.alteraPontosDeVida( pontosDeVida.getValue() );
			unidade.alteraPrecisao( precisao.getValue() );
			unidade.alteraPontosDeMovimento( 
                                      pontosDeMovimento.getValue() );
			for( int i = 0; i < modeloDisponiveis.size(); i++ ) {
                            unidade.removeAcao( 
                                ( AcaoAbstrata )modeloDisponiveis.get( i ) );
                        }
                        for( int i = 0; i < modeloEscolhidos.size(); i++ ) {
                            unidade.adicionaAcao( 
                                ( AcaoAbstrata )modeloEscolhidos.get( i ) );
			}
		    }
		    hide();
                }
            } );

        painelConfirma.add( botaoCancelar );
        painelConfirma.add( botaoConfirmar );

        Container c = getContentPane();
        
        c.setLayout( new BorderLayout() );
        c.add( painelAtributos, BorderLayout.NORTH );
        c.add( painelAcoes, BorderLayout.CENTER );
        c.add( painelConfirma, BorderLayout.SOUTH );
    }
    
    public void inicializaComponentes( Unidade unidade ) {
    
        inicializaComponentes();
        
        nome.setText( unidade.recuperaNome() );
                
        pontosDeVida.setValue( unidade.recuperaPontosDeVida() );
        
        precisao.setValue( unidade.recuperaPrecisao() );
        
        pontosDeMovimento.setValue( unidade.recuperaPontosDeMovimento() );
        
        String f = unidade.recuperaFigura();
        if ( f == null ) {
            figura.setSelectedItem( "" );
        }
        else {
            figura.setSelectedItem( f );
        }

        AcaoAbstrata[] acoes = unidade.recuperaAcoes();
        
        for( int i = 0; i < acoes.length; i ++ ) {
            modeloEscolhidos.addElement( acoes[i] );
            modeloDisponiveis.removeElement( acoes[i] );
        }
    }

    public Unidade recuperaUnidade() {
	
	return unidade;
    }

    public static void main( String[] main ) {
        
        Unidade viviUni = new Unidade( "vivi", 10, 10, 10, "vivisit.gif" );
        viviUni.adicionaAcao( Gerenciador.recuperaInstancia()
                              .recuperaAcoes()[0] );
        JanelaUnidade pu = new JanelaUnidade( null, viviUni );
        pu.show();
    }
}
