import java.io.File;
import java.util.ArrayList;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

public class PainelCriacaoMapa extends JPanel implements Observador {
    
    private CampoNumeroInteiro x;

    private CampoNumeroInteiro y;

    private CampoNumeroInteiro custo;
    
    private JComboBox fundo;

    private Setor[][] mapa;

    private PainelControleMapa pcm;

    public PainelCriacaoMapa( int linhas, int colunas ) {
    
        super( new BorderLayout() ) ;
        inicializaComponentes( linhas, colunas );
    }

    private void inicializaComponentes( int linhas, int colunas ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( linhas, colunas );
        
        pcm = new PainelControleMapa( mapa );
        pcm.setBorder( BorderFactory.createEtchedBorder() );
        pcm.adicionaObservador( this );

        ArrayList permitidos = new ArrayList();
        for( int i = 0; i < linhas; i++ ) {
            for( int j = 0; j < colunas; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }
        pcm.alteraPermitidos( permitidos );

        JPanel painelControle = new JPanel( new GridBagLayout() );
        painelControle.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        GridBagConstraints gbc = new GridBagConstraints();

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridx = 0;
        gbc.weightx = 1;
        gbc.gridy = 0;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.NORTH;

        JPanel aux = new JPanel( new GridLayout( 2, 1 ) );

        aux.add( new JLabel( "x:" ) );
        x = new CampoNumeroInteiro( 3 );
        aux.add( x );
        painelControle.add( aux, gbc );

        gbc.gridy = 1;
        
        aux = new JPanel( new GridLayout( 2, 1 ) );
        
        aux.add( new JLabel( "y:" ) );
        y = new CampoNumeroInteiro( 3 );
        aux.add( y );
        painelControle.add( aux, gbc );

        gbc.gridy = 2;

        aux = new JPanel( new GridLayout( 2, 1 ) );
        
        aux.add( new JLabel( "custo:" ) );
        custo = new CampoNumeroInteiro( 3 );
        aux.add( custo );
        painelControle.add( aux, gbc );
        
        gbc.gridy = 3;

        aux = new JPanel( new GridLayout( 2, 1 ) );

        File imagens = new File( "imagens" + File.separator + "fundos" );
        if( imagens.isDirectory() ) {
            fundo = new JComboBox( imagens.list() );
        }
        else {
            fundo = new JComboBox();
        }
        fundo.addItem( "" );
        fundo.setSelectedItem( "" );
        
        aux.add( new JLabel( "fundo:" ) );
        
        aux.add( fundo );
        painelControle.add( aux, gbc );
        
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridy = 4;
        gbc.weighty = 4;
        painelControle.add( new JPanel(), gbc );

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridx = 0;
        gbc.weightx = 1;
        gbc.gridy = 5;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.SOUTH;

        JButton botaoConfirma = new JButton( "confirma" );
        botaoConfirma.addActionListener( new ActionListener() {
                
                public void actionPerformed( ActionEvent ae ) {
                    
                    int valX = x.getValue();
                    int valY = y.getValue();
                    if( valX >= 0 && valX < mapa[0].length &&
                        valY >= 0 && valY < mapa.length       ) {
                        mapa[valY][valX]
                            .alteraCustoDeMovimento( custo.getValue() );
                        String f = ( String )fundo.getSelectedItem();
                        if( !f.equals( "" ) ) {
                            mapa[valY][valX].alteraFundo( f );
                        }
                        else {
                            mapa[valY][valX].alteraFundo( null );
                        }
                    }
                    pcm.centraliza( valX, valY );
                }
            } );
        painelControle.add( botaoConfirma, gbc );

        JPanel painelProximo = new JPanel( new BorderLayout() );

        JButton botaoProximo = new JButton( "Prosseguir" );
        botaoProximo.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    int[][] custos = new int[12][12];
                    String[][] fundos = new String[12][12];
                    String[] f = new String[3];
                    f[0] = "pedra.png";
                    f[1] = "agua.png";
                    f[2] = "grama.png";
                    java.util.Random r = new java.util.Random();
                    for( int i = 0; i < 12; i++ ) {
                        for( int j = 0; j < 12 ; j ++ ) {
                            custos[i][j] = r.nextInt( 5 ) + 1;
                            fundos[i][j] = f[ r.nextInt( 3 ) ];
                        } 
                    }
                    alteraMapa( custos, fundos );
                }
            } );

        painelProximo.add( botaoProximo, BorderLayout.EAST );
        painelProximo.setBorder( BorderFactory.createEtchedBorder() );

        add( pcm, BorderLayout.CENTER );
        add( painelControle, BorderLayout.EAST );
        add( painelProximo, BorderLayout.SOUTH );
        
    }

    public void atualiza( Object estado ) {
        
        Coordenada coord;
        try {
            coord = ( Coordenada ) estado;
        } catch( ClassCastException cce ) {
            return;
        }
        
        int valX = coord.recuperaX();
        int valY = coord.recuperaY();
        
        x.setValue( valX );
        y.setValue( valY );
        custo.setValue( mapa[valY][valX].recuperaCustoDeMovimento() );
        String f = mapa[valY][valX].recuperaFundo();
        if( f == null ) {
            fundo.setSelectedItem( "" );
        }
        else {
            fundo.setSelectedItem( f );
        }
    }

    public void novoMapa( int linhas, int colunas ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( linhas, colunas );
        pcm.alteraMapa( mapa );
        ArrayList permitidos = new ArrayList();
        for( int i = 0; i < linhas; i++ ) {
            for( int j = 0; j < colunas; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }                                 
        pcm.alteraPermitidos( permitidos );
        pcm.centraliza( 5, 3 );
    }

    public void alteraMapa( int[][] custos, String[][] fundos ) {
        
        mapa = Gerenciador.recuperaInstancia().geraMapa( custos, fundos );
        ArrayList permitidos = new ArrayList();
        pcm.alteraMapa( mapa );
        for( int i = 0; i < custos.length; i++ ) {
            for( int j = 0; j < custos[0].length; j++ ) {
                permitidos.add( new Coordenada( j, i ) );
            }
        }                                 
        pcm.alteraPermitidos( permitidos );
        pcm.centraliza( 5, 3 );
    }

    public static void main( String[] args ) {
        
        javax.swing.JFrame jf = new javax.swing.JFrame();
        jf.getContentPane().add( new PainelCriacaoMapa( 15 , 10 ) );
        jf.pack();
        jf.show();
    }
}
