import java.awt.*;
import javax.swing.*;

/**
 * A classe <code>PainelFoto</code>  responsvel por apresentar a imagem  
 * referenciada em foto
 * @author Grupo 4 - DOS
 * @version 1.0
 * @see PainelDetalhes
 * @see PainelLogo
 */
public class PainelFoto extends JPanel {

    /** Imagem a ser apresentada no painel */
    private Image foto;

     /**
     * Cria um novo objeto <code>PainelFoto</code>.
     */
    public PainelFoto() {
                
        foto = null;
    }

    /**
     * Cria um novo objeto <code>PainelFoto</code>.
     * @param caminho caminho relativo do arquivo referente a imagem 
     * desejada
     */
    public PainelFoto( String caminho ) {
                
        Toolkit tk = Toolkit.getDefaultToolkit();
        if( caminho != null ) {
            foto = tk.getImage( caminho );
        }
        else{
            foto = null;
        }
    }

    /**
     * modifica a imagem a ser apresentada pelo painel
     * @param caminho caminho relativo do arquivo referente a imagem 
     * desejada
     */
    public void atualizarFoto( String caminho ) {

        Toolkit tk = Toolkit.getDefaultToolkit();
        if( caminho != null ) {
            foto = tk.getImage( caminho );
        }                
        else {
            foto = null;
        }
    }           

    /**
     * metodo de pintura do painel
     * @param g objeto de pintura
     */
    protected void paintComponent( Graphics g ) {    
        if( foto != null ) {
            int h1, h2, w1, w2;
            h1 = this.getHeight();
            w1 = this.getWidth();
            h2 = ( int ) w1 * 
                foto.getHeight( this ) / foto.getWidth( this );
            w2 = ( int ) h1 *
                foto.getWidth( this ) / foto.getHeight( this );;
            if( h1 > h2 ){
                g.drawImage( foto, 0, ( h1 - h2 ) / 2, w1, h2, this );
            } 
            else{
                g.drawImage( foto, ( w1 - w2 ) / 2, 0, w2, h1, this );
            }
        }
    }

    /**
     * metodo de atualizacao da pintura do painel
     */
    protected void update(){
        paintComponent( getGraphics() );
    }
}
