import java.util.ArrayList;
import java.util.Collection;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BorderFactory;

public class PainelMapa extends JPanel implements Observado {

    private int origemX;
    private int origemY;

    private int limiteX;
    private int limiteY;

    private Setor[][] mapa;
    
    private ViewSetor[][] setores;

    private ArrayList setoresPermitidos;

    private ArrayList setoresAfetados;

    private ArrayList observadores;

    public PainelMapa( Setor[][] mapa ) {
        
        super( new GridLayout( 7, 9 ) );
        this.limiteX = mapa[0].length;
        this.limiteY = mapa.length;
	this.mapa = mapa;
        
        inicializaComponentes();

        setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        setPreferredSize( new Dimension( 360, 280 ) );
    }
    
    public void inicializaComponentes() {
        
        origemX = 0;
        origemY = 0;
        
        setores = new ViewSetor[7][9];
  
        for( int i = 0; i < 7 ; i++ ) {
            for( int j = 0; j < 9 ; j++ ) {
                setores[i][j] = new ViewSetor();
                setores[i][j].setBorder( 
                              BorderFactory.createLineBorder( Color.black, 1 ) 
                                        );
		setores[i][j].addMouseListener( new TratamentoSetor( j, i ) );
                add( setores[i][j] );
            }
        }

	setoresPermitidos = new ArrayList();
	setoresAfetados = new ArrayList();
	setoresAfetados.add( new Coordenada( 0, 0 ) );

        observadores = new ArrayList();
	
	atualiza();
    }

    public void alteraPermitidos( Collection permitidos ) {
	
	setoresPermitidos = new ArrayList( permitidos );
	atualiza();
    }

    public void alteraAfetados( Collection afetados ) {
	
        if( afetados.isEmpty() ) {
            afetados.add( new Coordenada( 0, 0 ) );
        }
        setoresAfetados = new ArrayList( afetados );
    }
    
    public void adicionaObservador( Observador observador ) {
    
        observadores.add( observador );
    }

    public void removeObservador( Observador observador ) {
    
        int i = observadores.indexOf( observador );
        if( i != -1 ) {
            observadores.remove( i );
        }
    }

    public void alteraMapa( Setor[][] novoMapa ) {
        
        this.mapa = novoMapa;
        this.limiteX = mapa[0].length;
        this.limiteY = mapa.length;
    }

    protected void notificaObservadores( Object estado ) {

        Observador observador;
        for( int i = 0; i < observadores.size(); i++ ) {
            observador = ( Observador ) observadores.get( i );
            observador.atualiza( estado );
        } 
    }

    private void atualiza() {
        
	Setor setor;
	Unidade unidade;
	for( int i = 0; i < 7 ; i++ ) {
	    for( int j = 0; j < 9 ; j++ ) {
		setores[i][j].permitido( false ); 
	    }
	}

	for( int k = 0; k < setoresPermitidos.size(); k++ ) {
	    Coordenada coord = ( Coordenada ) setoresPermitidos.get( k );
	    int x = coord.recuperaX() - origemX;
	    int y = coord.recuperaY() - origemY;
	    if( x >= 0 && x < 9 && y >=0 && y < 7 ) {
		setores[y][x].permitido( true );
	    }
	}

        for( int i = 0; i < 7 ; i++ ) {
            if( origemY + i >= 0 && origemY + i < limiteY) {
                for( int j = 0; j < 9 ; j++ ) {
                    if( origemX + j >= 0 && origemX + j < limiteX ) {
			setor = this.mapa[origemY + i][origemX + j];
			unidade = setor.recuperaUnidade();
			if( unidade != null ) {
			    setores[i][j].atualiza( setor.recuperaFundo(),
						    unidade.recuperaFigura() );
			}
			else {
			    setores[i][j].atualiza( setor.recuperaFundo(),
						    null );
			}
                    }
                    else{
                        setores[i][j].atualiza( null,null );
                    }
                }
            }
            else {
                for( int j = 0; j < 9 ; j++ ) {
		    setores[i][j].atualiza( null,null );
                }
            }
        }
    }

    public void alteraOrigemX( int x ) {
        
        this.origemX = x;
        atualiza();
    }

    public void ajustaOrigemX( int ajuste ) {
        
        this.origemX += ajuste;
        atualiza();
    }

    public void alteraOrigemY( int y ) {
        
        this.origemY = y;
        atualiza();
    }
            
    public void ajustaOrigemY( int ajuste ) {
        
        this.origemY += ajuste;
        atualiza();
    }

    public void centraliza( int x, int y ) {
        
        this.alteraOrigemX( x - 4 );
        this.alteraOrigemY( y - 3 );
        atualiza();
    }

    protected class TratamentoSetor extends MouseAdapter {
	
	private final int x;
	private final int y;

	public TratamentoSetor( int x, int y ) {
	    
	    this.x = x;
	    this.y = y;
	}

	public void mouseClicked( MouseEvent me ) {
            
            if( setores[this.y][this.x].estaPermitido() ) {
                notificaObservadores( new Coordenada( this.x + origemX,
                                                      this.y + origemY ) );
                int posX, posY;
		Coordenada coord;
		for( int i = 0; i < setoresAfetados.size(); i++ ) {
		    coord = ( Coordenada ) setoresAfetados.get( i );
		    posX = this.x + coord.recuperaX();
		    posY = this.y + coord.recuperaY();
		    if( posX >= 0 && posX < 9 && posY >=0 && posY < 7 ) {
			setores[posY][posX].afetado( false );
			setores[posY][posX].atualiza();
		    }
		}
            }
        }

	public void mouseEntered( MouseEvent me ) {
	    
	    if( setores[this.y][this.x].estaPermitido() ) {
		int posX, posY;
		Coordenada coord;
		for( int i = 0; i < setoresAfetados.size(); i++ ) {
		    coord = ( Coordenada ) setoresAfetados.get( i );
		    posX = this.x + coord.recuperaX();
		    posY = this.y + coord.recuperaY();
		    if( posX >= 0 && posX < 9 && posY >=0 && posY < 7 ) {
			setores[posY][posX].afetado( true );
			setores[posY][posX].atualiza();
		    }
		}
	    }
	}

	public void mouseExited( MouseEvent me ) {
	
	    if( setores[this.y][this.x].estaPermitido() ) {
		int posX, posY;
		Coordenada coord;
		for( int i = 0; i < setoresAfetados.size(); i++ ) {
		    coord = ( Coordenada ) setoresAfetados.get( i );
		    posX = this.x + coord.recuperaX();
		    posY = this.y + coord.recuperaY();
		    if( posX >= 0 && posX < 9 && posY >=0 && posY < 7 ) {
			setores[posY][posX].afetado( false );
			setores[posY][posX].atualiza();
		    }
		}
	    }
	}
    }
}



