import java.io.File;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

public class PainelUnidade extends JPanel {

    private JTextField nome;
    private CampoNumeroInteiro pontosDeVida;
    private CampoNumeroInteiro precisao;
    private CampoNumeroInteiro pontosDeMovimento;
    
    private Unidade unidade;
    
    private DefaultListModel modeloDisponiveis;
    private JList acoesDisponiveis;
    private DefaultListModel modeloEscolhidos;
    private JList acoesEscolhidas;

    public PainelUnidade() {

        super( new BorderLayout() );
        inicializaComponentes();
    }

    public PainelUnidade( Unidade unidade ) {
        
        this.unidade = unidade;
        inicializaComponentes( unidade );
    }

    public void inicializaComponentes() {

        JPanel painelAtributos = new JPanel( new GridLayout( 5, 2 ) );

        nome = new JTextField( 5 );
        pontosDeVida = new CampoNumeroInteiro( 5 );
        precisao = new CampoNumeroInteiro( 5 );
        pontosDeMovimento = new CampoNumeroInteiro( 5 );

        painelAtributos.add( new JLabel( "Nome:" ) );
        painelAtributos.add( nome );

        painelAtributos.add( new JLabel( "Pts de Vida:" ) );
        painelAtributos.add( pontosDeVida );

        painelAtributos.add( new JLabel( "Preciso:" ) );
        painelAtributos.add( precisao );

        painelAtributos.add( new JLabel( "Pts de Movimento:" ) );
        painelAtributos.add( pontosDeMovimento );

        painelAtributos.add( new JLabel( "Figura:" ) );

        painelAtributos.setBorder( 
              BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        JPanel painelAcoes = new JPanel( new GridLayout( 1, 3 ) );

        modeloDisponiveis = new DefaultListModel();
        acoesDisponiveis = new JList( modeloDisponiveis );

        painelAcoes.add( new JScrollPane( acoesDisponiveis ) );

        AcaoAbstrata[] acoes = 
            Gerenciador.recuperaInstancia().recuperaAcoes();
        
        for( int i = 0; i< acoes.length; i++ ) {
            modeloDisponiveis.addElement( acoes[i] );
        }

        JPanel painelBotoes = new JPanel( new GridBagLayout() );
        
        GridBagConstraints gbc = new GridBagConstraints();     

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.gridx = 0;
        gbc.gridy = 0;

        JButton botaoIncluir = new JButton( "incluir" );

        botaoIncluir.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    Object[] acoes = acoesDisponiveis.getSelectedValues();
                    for( int i = 0; i < acoes.length; i++ ) {
                        modeloEscolhidos.addElement( acoes[i] );
                        modeloDisponiveis.removeElement( acoes[i] );
                    }
                }
            } ); 
        
        painelBotoes.add( botaoIncluir, gbc );

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.gridx = 0;
        gbc.gridy = 1;

        JButton botaoExcluir = new JButton( "excluir" );
        botaoExcluir.addActionListener( new ActionListener() {
            
                public void actionPerformed( ActionEvent ae ) {
                    
                    Object[] acoes = acoesEscolhidas.getSelectedValues();
                    for( int i = 0; i < acoes.length; i++ ) {
                        modeloDisponiveis.addElement( acoes[i] );
                        modeloEscolhidos.removeElement( acoes[i] );
                    }
                }
            } ); 
        painelBotoes.add( botaoExcluir, gbc );

        painelBotoes.setBorder( 
                      BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );

        painelAcoes.add( painelBotoes );

        modeloEscolhidos = new DefaultListModel();
        acoesEscolhidas = new JList( modeloEscolhidos );

        painelAcoes.add( new JScrollPane( acoesEscolhidas ) );

        painelAcoes.setBorder( 
              BorderFactory.createCompoundBorder( 
                    BorderFactory.createTitledBorder("Aes"),
                    BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        add( painelAtributos, BorderLayout.NORTH );
        add( painelAcoes, BorderLayout.CENTER );
    }
    
    public void inicializaComponentes( Unidade unidade ) {
    
        inicializaComponentes();
        nome.setText( unidade.recuperaNome() );
        nome.setEditable( false );
        pontosDeVida.setValue( unidade.recuperaPontosDeVida() );
        precisao.setValue( unidade.recuperaPrecisao() );
        pontosDeMovimento.setValue( unidade.recuperaPontosDeMovimento() );
    }

    public static void main( String[] main ) {
        
        javax.swing.JFrame jf = new javax.swing.JFrame();
        jf.getContentPane().add( new PainelUnidade() );
        jf.setSize( 400, 350 );
        jf.show();
    }
}
