import java.util.ArrayList;

/**
 * 
 * @version 1.0
 * @author Alex Minoru Kusano & Marcos Roberto Yukio Koga
 */
public class Unidade implements Observado, Visitado {

    private String nome;

    private Jogador jogador;

    /** quantidade de dano que a unidade suporta */
    private int pontosDeVida;

    /** atributo que influencia na chance de acerto de uma unidade */
    private int precisao;

    /** no' do mapa no qual a unidade se encontra */
    private Setor posicao;

    /** colecao de acoes que a unidade pode tomar */
    private ArrayList acoes;

    /** atributo que quantifica o quanto a unidade pode mover */
    private int pontosDeMovimento;

    private String figura;
    
    /** colecao de observadores */
    private ArrayList observadores;
    
    
    public Unidade ( String nome, 
                     int pontosDeVida, 
                     int precisao, 
		     int pontosDeMovimento,
		     String figura ) {
	
        this.nome = nome;
	this.pontosDeVida = pontosDeVida;
	this.precisao = precisao;
	this.pontosDeMovimento = pontosDeMovimento;
	this.figura = figura;
	
	acoes = new ArrayList();
	observadores = new ArrayList();
    }

    public String recuperaNome() {
        
        return this.nome;
    }

    public void alteraJogador( Jogador jogador ) {
	
	this.jogador = jogador;
    }

    public Jogador recuperaJogador() {

	return this.jogador;
    }
    
    public int recuperaPontosDeVida() {
	
	return this.pontosDeVida;
    }

    public void alteraPontosDeVida( int novaPontuacao ) {
    
	this.pontosDeVida = novaPontuacao;
    }

    public void ajustaPontosDeVida( int ajuste ) {
	
	this.pontosDeVida += ajuste;
	if( this.pontosDeVida < 1 ) {
	    notificaObservadores( this );
	    jogador.removeUnidade( this );
	    posicao.removeUnidade();
	}
    }

    public int recuperaPrecisao() {
	
	return this.precisao;
    }

    public void alteraPrecisao( int novaPrecisao ) {
	
	this.precisao = novaPrecisao;
    }

    public void ajustaPrecisao( int ajuste ) {
	
	this.precisao += ajuste;
    }

    public Setor recuperaPosicao() {
	
	return this.posicao; 
    }

    public void move( Setor posicao ) {
	
	this.posicao = posicao;
    }

    public int recuperaPontosDeMovimento() {
	
	return this.pontosDeMovimento;
    }

    public void alteraPontosDeMovimento( int novaPontuacao ) {
    
	this.pontosDeMovimento = novaPontuacao;
    }

    public void ajustaPontosDeMovimento( int ajuste ) {
	
	this.pontosDeMovimento += ajuste;
    }

    public void alteraFigura( String novaFigura ) {
	
	this.figura = novaFigura;
    }

    public String recuperaFigura() {
	
	return this.figura;
    }

    public void adicionaAcao( AcaoAbstrata acao ) {
	
        if( ! acoes.contains( acao ) ) {
            acoes.add( acao );
        }
    }

    public void removeAcao( AcaoAbstrata acao ) {
    
	int i = acoes.indexOf( acao );
	if( i != -1 ) {
	    acoes.remove( i );
	}
    }

    public AcaoAbstrata[] recuperaAcoes() {
	
	AcaoAbstrata[] retorno =  new AcaoAbstrata[ this.acoes.size() ];
	this.acoes.toArray( retorno );
	return retorno;
    }

    public void adicionaObservador( Observador observador ) {
	
	observadores.add( observador );
    }

    public void removeObservador( Observador observador ) {
    
	int i = observadores.indexOf( observador );
	if( i != -1 ) {
	    observadores.remove( i );
	}
    }

    protected void notificaObservadores( Object estado ) {
        
	Observador observador;
	for( int i = 0; i < observadores.size(); i++ ) {
	    observador = ( Observador ) observadores.get( i );
	    observador.atualiza( estado );
	}
    }

    public void executaAcao( AcaoAbstrata acao ) {
	
	acao.associaAgente( this );
    }

    public void sofreAcao( AcaoAbstrata acao ) {
    
	acao.executa( this );
    }
 
    public Object clone() {
	
	Unidade clone =  new Unidade( this.nome,
                                      this.pontosDeVida, 
                                      this.precisao, 
				      this.pontosDeMovimento,
				      this.figura );
	for( int i = 0; i < acoes.size(); i++ ) {
	    clone.adicionaAcao( ( AcaoAbstrata )acoes.get( i ) );
	}
	return ( Object ) clone;
    }

    public String toString() {
	
	return nome;
    }

    public boolean equals( Object objeto ) {
        
        if( ! getClass().isInstance( objeto ) ) {
            return false;
        }
        
        Unidade unidade = ( Unidade )objeto;

        if( this.nome.equals( unidade.recuperaNome() ) ){

	    if( posicao != null )
		return posicao.equals( unidade.recuperaPosicao() );
	    else
		return true;
	}

	return false;
    }
}


